@@ -1,5 +1,8 @@
+import re
 from datetime import datetime, timezone
-from typing import Any, Dict, Optional
+from typing import Any, Dict, Optional, Mapping
+import collections.abc
+import copy
 import warnings
 
 from functools import singledispatch
@@ -11,7 +14,13 @@ from pystac.utils import datetime_to_str
 from pystac_client import ItemSearch
 
 from planetary_computer.settings import Settings
-from planetary_computer.utils import parse_blob_url, parse_adlfs_url, is_fsspec_asset
+from planetary_computer.utils import (
+    parse_blob_url,
+    parse_adlfs_url,
+    is_fsspec_asset,
+    is_vrt_string,
+    asset_xpr,
+)
 
 
 BLOB_STORAGE_DOMAIN = ".blob.core.windows.net"
@@ -63,23 +72,52 @@ def sign(obj: Any) -> Any:
 
     Args:
         obj (Any): The object to sign. Must be one of:
-            str (URL), Asset, Item, ItemCollection, or ItemSearch
+            str (URL), Asset, Item, ItemCollection, or ItemSearch, or a mapping.
     Returns:
         Any: A copy of the object where all relevant URLs have been signed
     """
     raise TypeError(
-        "Invalid type, must be one of: str, Asset, Item, ItemCollection, or ItemSearch"
+        "Invalid type, must be one of: str, Asset, Item, ItemCollection, "
+        "ItemSearch, or mapping"
     )
 
 
 @sign.register(str)
-def sign_url(url: str) -> str:
-    """Sign a URL with a Shared Access (SAS) Token, which allows for read access.
+def sign_string(url: str) -> str:
+    """Sign a URL or VRT-like string containing URLs with a Shared Access (SAS) Token
+
+    Signing with a SAS token allows read access to files in blob storage.
 
     Args:
-        url (str): The HREF of the asset in the format of a URL.
-            This can be found on STAC Item's Asset 'href' value. Only URLs to assets
-            in Azure Blob Storage are signed, other URLs are returned unmodified.
+        url (str): The HREF of the asset as a URL or a GDAL VRT
+
+            Single URLs can be found on a STAC Item's Asset ``href`` value. Only URLs to
+            assets in Azure Blob Storage are signed, other URLs are returned unmodified.
+
+            GDAL VRTs can combine many data sources into a single mosaic. A VRT can be
+            built quickly from the GDAL STACIT driver
+            https://gdal.org/drivers/raster/stacit.html. Each URL to Azure Blob Storage
+            within the VRT is signed.
+
+    Returns:
+        str: The signed HREF or VRT
+    """
+    if is_vrt_string(url):
+        return sign_vrt_string(url)
+    else:
+        return sign_url(url)
+
+
+def sign_url(url: str) -> str:
+    """Sign a URL or with a Shared Access (SAS) Token
+
+    Signing with a SAS token allows read access to files in blob storage.
+
+    Args:
+        url (str): The HREF of the asset as a URL
+
+            Single URLs can be found on a STAC Item's Asset ``href`` value. Only URLs to
+            assets in Azure Blob Storage are signed, other URLs are returned unmodified.
 
     Returns:
         str: The signed HREF
@@ -93,6 +131,46 @@ def sign_url(url: str) -> str:
     return token.sign(url).href
 
 
+def _repl_vrt(m: re.Match) -> str:
+    # replace all blob-storages URLs with a signed version.
+    return sign_url(m.string[slice(*m.span())])
+
+
+def sign_vrt_string(vrt: str) -> str:
+    """Sign a VRT-like string containing URLs with a Shared Access (SAS) Token
+
+    Signing with a SAS token allows read access to files in blob storage.
+
+    Args:
+        vrt (str): The GDAL VRT
+
+            GDAL VRTs can combine many data sources into a single mosaic. A VRT can be
+            built quickly from the GDAL STACIT driver
+            https://gdal.org/drivers/raster/stacit.html. Each URL to Azure Blob Storage
+            within the VRT is signed.
+
+    Returns:
+        str: The signed VRT
+
+    Examples
+    --------
+    >>> from osgeo import gdal
+    >>> from pathlib import Path
+    >>> search = (
+    ...     "STACIT:\"https://planetarycomputer.microsoft.com/api/stac/v1/search?"
+    ...     "collections=naip&bbox=-100,40,-99,41"
+    ...     "&datetime=2019-01-01T00:00:00Z%2F..\":asset=image"
+    ... )
+    >>> gdal.Translate("out.vrt", search)
+    >>> signed_vrt = planetary_computer.sign(Path("out.vrt").read_text())
+    >>> print(signed_vrt)
+    <VRTDataset rasterXSize="161196" rasterYSize="25023">
+    ...
+    </VRTDataset>
+    """
+    return asset_xpr.sub(_repl_vrt, vrt)
+
+
 @sign.register(Item)
 def sign_item(item: Item) -> Item:
     """Sign all assets within a PySTAC item
@@ -140,19 +218,31 @@ def _sign_asset_in_place(asset: Asset) -> Asset:
     if is_fsspec_asset(asset):
         key: Optional[str]
 
+        storage_options = None
         for key in ["table:storage_options", "xarray:storage_options"]:
             if key in asset.extra_fields:
+                storage_options = asset.extra_fields[key]
                 break
-        else:
-            key = None
+        if storage_options is None:
+            storage_options = asset.extra_fields.get("xarray:open_kwargs", {}).get(
+                "storage_options", None
+            )
+
+        if storage_options is None:
+            storage_options = (
+                asset.extra_fields.get("xarray:open_kwargs", {})
+                .get("backend_kwargs", {})
+                .get("storage_options", None)
+            )
+
+        if storage_options is None:
+            return asset
+        account = storage_options.get("account_name")
+        container = parse_adlfs_url(asset.href)
+        if account and container:
+            token = get_token(account, container)
+            storage_options["credential"] = token.token
 
-        if key:
-            storage_options = asset.extra_fields[key]
-            account = storage_options.get("account_name")
-            container = parse_adlfs_url(asset.href)
-            if account and container:
-                token = get_token(account, container)
-                asset.extra_fields[key]["credential"] = token.token
     return asset
 
 
@@ -179,12 +269,11 @@ def sign_item_collection(item_collection: ItemCollection) -> ItemCollection:
         a "msft:expiry" property is added to the Item properties indicating the
         earliest expiry time for any assets that were signed.
     """
-    return ItemCollection.from_dict(
-        {
-            "type": "FeatureCollection",
-            "features": [sign(item).to_dict() for item in item_collection],
-        }
-    )
+    new = item_collection.clone()
+    for item in new:
+        for key in item.assets:
+            _sign_asset_in_place(item.assets[key])
+    return new
 
 
 @sign.register(ItemSearch)
@@ -203,6 +292,38 @@ def _search_and_sign(search: ItemSearch) -> ItemCollection:
     return sign(search.get_all_items())
 
 
+@sign.register(collections.abc.Mapping)
+def sign_reference_file(references: Mapping) -> Mapping:
+    """
+    Sign a Kerchunk-style dictionary of references.
+
+    Args:
+        references (Mapping): The dictionary containing the references.
+
+    Returns:
+        references (Mapping): The dictionary, now with signed URLs.
+
+    This method will sign all of the URLs under the ``"templates"`` key. Mappings
+    with other keys will be returned unchanged. See https://fsspec.github.io/kerchunk/
+    for more.
+    """
+    # TODO(python3.8): Use TypedDict instead of Mapping to limit the kinds we accept.
+    # If we accept other mapping types in the future, we'll need to dispatch within
+    # the function.
+    if set(references) == {"version", "templates", "refs"}:
+        references = copy.deepcopy(references)
+
+        for k, v in references["templates"].items():
+            references["templates"][k] = sign_url(v)
+
+    else:
+        raise TypeError(
+            "When providing a mapping, the keys should be 'version', 'templates', "
+            "and 'refs'."
+        )
+    return references
+
+
 def get_token(account_name: str, container_name: str) -> SASToken:
     """
     Get a token for a container in a storage account.
